﻿namespace U2UConsult.EF40.TransactionScope.Sample
{
    using System;
    using System.Data.Common;
    using System.Diagnostics;
    using System.ServiceProcess;
    using System.Transactions;
    using System.Windows;
    using System.Windows.Input;
    using System.Windows.Media;
    using System.Windows.Threading;

    /// <summary>
    /// Demo on Transaction and Connection settings in Entity Framework 4.0.
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            DispatcherTimer timer = new DispatcherTimer(
                TimeSpan.FromSeconds(2),
                DispatcherPriority.Normal,
                this.UpdateMsdtcStatus,
                this.Dispatcher);

            App.Current.DispatcherUnhandledException += this.Current_DispatcherUnhandledException;
        }

        /// <summary>
        /// Handles unhandled exceptions.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Current_DispatcherUnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            Mouse.OverrideCursor = null;

            this.ErrorLabel.Content = e.Exception.Message;

            if (e.Exception.InnerException != null)
            {
                this.ErrorLabel.Content += " " + e.Exception.InnerException.Message;
            }

            e.Handled = true;
        }

        /// <summary>
        /// Updates the MSDTC status.
        /// </summary>
        /// <param name="sender">Sender of the event (Timer).</param>
        /// <param name="e">Event arguments.</param>
        private void UpdateMsdtcStatus(object sender, EventArgs e)
        {
            ServiceController sc = new ServiceController("Distributed Transaction Coordinator");
            this.msdtcStatusLabel.Text = "MSDTC Status: " + sc.Status.ToString();
        }

        /// <summary>
        /// Executes a DAL method, with default settings.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void FirstButton_Click(object sender, RoutedEventArgs e)
        {
            this.DefaultLabel.Text = this.DoSomeStuff(false, true) + " ms.";
        }

        /// <summary>
        /// Executes a DAL method, with explicit connection opening.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void SecondButton_Click(object sender, RoutedEventArgs e)
        {
            this.OpenLabel.Text = this.DoSomeStuff(true, true) + " ms.";
        }

        /// <summary>
        /// Executes a DAL method, without connection pooling.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void ThirdButton_Click(object sender, RoutedEventArgs e)
        {
            this.DefaultLabel2.Foreground = Brushes.WhiteSmoke;

            this.DefaultLabel2.Text = this.DoSomeStuff(false, false) + " ms.";
        }

        /// <summary>
        /// Executes a DAL method, without connection pooling, but with explicit connection opening.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void FourthButton_Click(object sender, RoutedEventArgs e)
        {
            this.OpenLabel2.Text = this.DoSomeStuff(true, false) + " ms.";
        }

        /// <summary>
        /// Starts the MSDTC service on the local computer.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void StartMsdtcButton_Click(object sender, RoutedEventArgs e)
        {
            this.ErrorLabel.Content = null;

            using (ServiceController sc = new ServiceController("Distributed Transaction Coordinator"))
            {
                if (sc.Status != ServiceControllerStatus.Running)
                {
                    sc.Start();
                }
            }
        }

        /// <summary>
        /// Stops the MSDTC service on the local computer.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void StopMsdtcButton_Click(object sender, RoutedEventArgs e)
        {
            this.ErrorLabel.Content = null;

            using (ServiceController sc = new ServiceController("Distributed Transaction Coordinator"))
            {
                if (sc.Status == ServiceControllerStatus.Running)
                {
                    sc.Stop(); // Admins only !!
                }
            }
        }

        /// <summary>
        /// Represents a DAL method.
        /// </summary>
        /// <param name="openConnectionExplicitely">Whether or not the connection should be opened explicitely.</param>
        /// <param name="enablePooling">Whether or not connection pooling is enabled.</param>
        /// <returns>The duration of the call, in milliseconds.</returns>
        private long DoSomeStuff(bool openConnectionExplicitely, bool enablePooling)
        {
            Mouse.OverrideCursor = Cursors.Wait;

            this.ErrorLabel.Content = null;

            string entitiesConnectionstring = "name=AdventureWorks2008Entities";
            if (!enablePooling)
            {
                entitiesConnectionstring += "NoPooling";
            }

            TransactionOptions options = new TransactionOptions();
            options.IsolationLevel = IsolationLevel.RepeatableRead;

            Stopwatch sw = new Stopwatch();
            sw.Start();

            using (System.Transactions.TransactionScope outerscope = new System.Transactions.TransactionScope(TransactionScopeOption.RequiresNew, options))
            {
                using (AdventureWorks2008Entities entities = new AdventureWorks2008Entities(entitiesConnectionstring))
                {
                    if (openConnectionExplicitely)
                    {
                        entities.Connection.Open();
                    }

                    Culture klingonCulture = new Culture() { CultureID = "kl", Name = "Klingon", ModifiedDate = DateTime.Now };
                    Culture borgCulture = new Culture() { CultureID = "br", Name = "Borg", ModifiedDate = DateTime.Now };

                    for (int i = 0; i < 25; i++)
                    {
                        using (System.Transactions.TransactionScope scope = new System.Transactions.TransactionScope(TransactionScopeOption.Required, options))
                        {
                            entities.Cultures.AddObject(klingonCulture);
                            entities.Cultures.AddObject(borgCulture);

                            entities.SaveChanges();

                            scope.Complete();
                        }

                        // Changes the isolation level inside the transaction
                        // entities.ExecuteStoreCommand("SET TRANSACTION ISOLATION LEVEL READ COMMITTED");

                        // Might escalate to DTC:
                        // entities.Connection.Close(); 

                        klingonCulture.Name = "Klingonian";
                        borgCulture.Name = "Borgian";
                        entities.SaveChanges();

                        using (System.Transactions.TransactionScope scope2 = new System.Transactions.TransactionScope(TransactionScopeOption.Required))
                        {
                            entities.Cultures.Attach(klingonCulture);
                            entities.Cultures.Attach(borgCulture);

                            entities.Cultures.DeleteObject(klingonCulture);
                            entities.Cultures.DeleteObject(borgCulture);

                            entities.SaveChanges();

                            scope2.Complete();
                        }
                    }

                    outerscope.Complete();
                }
            }

            sw.Stop();

            // Check isolation level
            // In the Data Access Layer
            using (AdventureWorks2008Entities entities = new AdventureWorks2008Entities(entitiesConnectionstring))
            {
                entities.Connection.Open();

                // Always returns 'Read Committed'
                DbTransaction trx = entities.Connection.BeginTransaction();
                this.ErrorLabel.Content = trx.IsolationLevel.ToString();

                // Your stuff here ...

                trx.Commit();
            }

            // Check isolation level 2
            // In the Business Layer
            using (System.Transactions.TransactionScope scope = new System.Transactions.TransactionScope(TransactionScopeOption.RequiresNew))
            {
                // In the Data Access Layer
                using (AdventureWorks2008Entities entities = new AdventureWorks2008Entities(entitiesConnectionstring))
                {
                    // Always returns 'Serializable'
                    this.ErrorLabel.Content = Transaction.Current.IsolationLevel.ToString();

                    // Your stuff here ...
                }

                scope.Complete();
            }

            // Check isolation level 3
            using (AdventureWorks2008Entities entities = new AdventureWorks2008Entities(entitiesConnectionstring))
            {
                // Script will reveal different values, depending on connection pooling
                // Uncomment for testing:
                // entities.ExecuteStoreCommand("WAITFOR DELAY '00:00:10'");
            }

            Mouse.OverrideCursor = null;

            return sw.ElapsedMilliseconds;
        }
    }
}